@@ -25,6 +25,7 @@
 #include "ephy-embed.h"
 #include "ephy-embed-shell.h"
 #include "ephy-embed-type-builtins.h"
+#include "ephy-evince-document-view.h"
 #include "ephy-file-helpers.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
@@ -42,6 +43,7 @@ struct _EphyDownload {
   char *content_type;
 
   gboolean show_notification;
+  gboolean in_document_mode;
 
   EphyDownloadActionType action;
   guint32 start_time;
@@ -630,6 +632,16 @@ download_decide_destination_cb (WebKitDownload *wk_download,
                                 const gchar    *suggested_filename,
                                 EphyDownload   *download)
 {
+  if (download->in_document_mode) {
+    g_autofree gchar *tmp_file = g_strdup_printf ("%s/%s", g_get_user_cache_dir (), suggested_filename);
+    g_autofree gchar *file_uri = g_filename_to_uri (tmp_file, NULL, NULL);
+
+    webkit_download_set_allow_overwrite (wk_download, TRUE);
+    webkit_download_set_destination (wk_download, file_uri);
+
+    return TRUE;
+  }
+
   if (webkit_download_get_destination (wk_download))
     return TRUE;
 
@@ -759,21 +771,21 @@ ephy_download_new (WebKitDownload *download)
 
   ephy_download = g_object_new (EPHY_TYPE_DOWNLOAD, NULL);
 
-  g_signal_connect_object (download, "notify::response",
-                           G_CALLBACK (download_response_changed_cb),
-                           ephy_download, 0);
-  g_signal_connect_object (download, "decide-destination",
-                           G_CALLBACK (download_decide_destination_cb),
-                           ephy_download, 0);
-  g_signal_connect_object (download, "created-destination",
-                           G_CALLBACK (download_created_destination_cb),
-                           ephy_download, 0);
-  g_signal_connect_object (download, "finished",
-                           G_CALLBACK (download_finished_cb),
-                           ephy_download, 0);
-  g_signal_connect_object (download, "failed",
-                           G_CALLBACK (download_failed_cb),
-                           ephy_download, 0);
+  g_signal_connect (download, "notify::response",
+                    G_CALLBACK (download_response_changed_cb),
+                    ephy_download);
+  g_signal_connect (download, "decide-destination",
+                    G_CALLBACK (download_decide_destination_cb),
+                    ephy_download);
+  g_signal_connect (download, "created-destination",
+                    G_CALLBACK (download_created_destination_cb),
+                    ephy_download);
+  g_signal_connect (download, "finished",
+                    G_CALLBACK (download_finished_cb),
+                    ephy_download);
+  g_signal_connect (download, "failed",
+                    G_CALLBACK (download_failed_cb),
+                    ephy_download);
 
   ephy_download->download = g_object_ref (download);
   g_object_set_data (G_OBJECT (download), "ephy-download-set", GINT_TO_POINTER (TRUE));
@@ -812,3 +824,9 @@ ephy_download_disable_desktop_notification (EphyDownload *download)
 
   download->show_notification = FALSE;
 }
+
+void
+ephy_download_enable_evince_document_mode (EphyDownload *download)
+{
+  download->in_document_mode = TRUE;
+}
