@@ -25,6 +25,7 @@
 #include "ephy-about-handler.h"
 #include "ephy-dbus-util.h"
 #include "ephy-debug.h"
+#include "ephy-downloads-manager.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-prefs.h"
 #include "ephy-embed-type-builtins.h"
@@ -755,6 +756,7 @@ ephy_embed_shell_get_global_history_service (EphyEmbedShell *shell)
     EphySQLiteConnectionMode mode;
 
     if (priv->mode == EPHY_EMBED_SHELL_MODE_INCOGNITO ||
+        priv->mode == EPHY_EMBED_SHELL_MODE_AUTOMATION ||
         priv->mode == EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER)
       mode = EPHY_SQLITE_CONNECTION_MODE_READ_ONLY;
     else
@@ -957,7 +959,7 @@ initialize_web_extensions (WebKitWebContext *web_context,
 
   address = priv->dbus_server ? g_dbus_server_get_client_address (priv->dbus_server) : NULL;
 
-  private_profile = priv->mode == EPHY_EMBED_SHELL_MODE_PRIVATE || priv->mode == EPHY_EMBED_SHELL_MODE_INCOGNITO;
+  private_profile = priv->mode == EPHY_EMBED_SHELL_MODE_PRIVATE || priv->mode == EPHY_EMBED_SHELL_MODE_INCOGNITO || priv->mode == EPHY_EMBED_SHELL_MODE_AUTOMATION;
   browser_mode = priv->mode == EPHY_EMBED_SHELL_MODE_BROWSER;
   user_data = g_variant_new ("(smsssbb)",
                              priv->guid,
@@ -1100,6 +1102,12 @@ ephy_embed_shell_create_web_context (EphyEmbedShell *shell)
     return;
   }
 
+  if (priv->mode == EPHY_EMBED_SHELL_MODE_AUTOMATION) {
+    priv->web_context = webkit_web_context_new_ephemeral ();
+    webkit_web_context_set_automation_allowed (priv->web_context, TRUE);
+    return;
+  }
+
   data_dir = g_build_filename (priv->mode == EPHY_EMBED_SHELL_MODE_PRIVATE ?
                                ephy_dot_dir () : g_get_user_data_dir (),
                                g_get_prgname (), NULL);
@@ -1134,6 +1142,44 @@ adblock_filters_dir (EphyEmbedShell *shell)
   return result;
 }
 
+static void
+download_started_cb (WebKitWebContext *web_context,
+                     WebKitDownload   *download,
+                     EphyEmbedShell   *shell)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+  EphyDownload *ephy_download;
+  EphyEmbed *embed;
+  GtkWindow *window;
+  gboolean ephy_download_set;
+
+  /* Is download locked down? */
+  if (g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
+                              EPHY_PREFS_LOCKDOWN_SAVE_TO_DISK)) {
+    webkit_download_cancel (download);
+    return;
+  }
+
+  /* Only create an EphyDownload for the WebKitDownload if it doesn't exist yet.
+   * This can happen when the download has been started automatically by WebKit,
+   * due to a context menu action or policy checker decision. Downloads started
+   * explicitly by Epiphany are marked with ephy-download-set GObject data.
+   */
+  ephy_download_set = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (download), "ephy-download-set"));
+  if (ephy_download_set)
+    return;
+
+  ephy_download = ephy_download_new (download);
+
+  window = gtk_application_get_active_window (GTK_APPLICATION (shell));
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  if (embed != NULL)
+    ephy_embed_download_started (embed, ephy_download);
+
+  ephy_downloads_manager_add_download (priv->downloads_manager, ephy_download);
+  g_object_unref (ephy_download);
+}
+
 static void
 ephy_embed_shell_startup (GApplication *application)
 {
@@ -1227,17 +1273,21 @@ ephy_embed_shell_startup (GApplication *application)
 
   priv->password_manager = ephy_password_manager_new ();
 
-  /* Favicon Database */
-  if (priv->mode == EPHY_EMBED_SHELL_MODE_PRIVATE)
-    favicon_db_path = g_build_filename (ephy_dot_dir (), "icondatabase", NULL);
-  else
-    favicon_db_path = g_build_filename (g_get_user_cache_dir (), "epiphany", "icondatabase", NULL);
-  webkit_web_context_set_favicon_database_directory (priv->web_context, favicon_db_path);
-  g_free (favicon_db_path);
-
-  /* Do not ignore TLS errors. */
-  webkit_web_context_set_tls_errors_policy (priv->web_context, WEBKIT_TLS_ERRORS_POLICY_FAIL);
+  /* Do not cache favicons in automation mode. Don't change the TLS policy either, since that's
+   * handled by session capabilities in automation mode.
+   */
+  if (priv->mode != EPHY_EMBED_SHELL_MODE_AUTOMATION) {
+    /* Favicon Database */
+    if (priv->mode == EPHY_EMBED_SHELL_MODE_PRIVATE)
+      favicon_db_path = g_build_filename (ephy_dot_dir (), "icondatabase", NULL);
+    else
+      favicon_db_path = g_build_filename (g_get_user_cache_dir (), "epiphany", "icondatabase", NULL);
+    webkit_web_context_set_favicon_database_directory (priv->web_context, favicon_db_path);
+    g_free (favicon_db_path);
 
+    /* Do not ignore TLS errors. */
+    webkit_web_context_set_tls_errors_policy (priv->web_context, WEBKIT_TLS_ERRORS_POLICY_FAIL);
+  }
 
   /* about: URIs handler */
   priv->about_handler = ephy_about_handler_new ();
@@ -1268,7 +1318,7 @@ ephy_embed_shell_startup (GApplication *application)
 
   /* Store cookies in moz-compatible SQLite format */
   cookie_manager = webkit_web_context_get_cookie_manager (priv->web_context);
-  if (priv->mode != EPHY_EMBED_SHELL_MODE_INCOGNITO) {
+  if (!webkit_web_context_is_ephemeral (priv->web_context)) {
     filename = g_build_filename (ephy_dot_dir (), "cookies.sqlite", NULL);
     webkit_cookie_manager_set_persistent_storage (cookie_manager, filename,
                                                   WEBKIT_COOKIE_PERSISTENT_STORAGE_SQLITE);
@@ -1283,6 +1333,9 @@ ephy_embed_shell_startup (GApplication *application)
   filters_dir = adblock_filters_dir (shell);
   priv->filters_manager = ephy_filters_manager_new (filters_dir);
   g_free (filters_dir);
+
+  g_signal_connect (priv->web_context, "download-started",
+                    G_CALLBACK (download_started_cb), shell);
 }
 
 static void
@@ -1688,7 +1741,10 @@ ephy_embed_shell_launch_handler (EphyEmbedShell *shell,
   gboolean ret = FALSE;
 
   g_assert (EPHY_IS_EMBED_SHELL (shell));
-  g_assert (file || mime_type);
+  g_assert (file);
+
+  if (mime_type == NULL)
+    return ret;
 
   if (ephy_is_running_inside_flatpak ()) {
     return ephy_file_launch_file_via_uri_handler (file);
