@@ -46,7 +46,8 @@ typedef enum
   EPHY_EMBED_SHELL_MODE_INCOGNITO,
   EPHY_EMBED_SHELL_MODE_APPLICATION,
   EPHY_EMBED_SHELL_MODE_TEST,
-  EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER
+  EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER,
+  EPHY_EMBED_SHELL_MODE_AUTOMATION
 } EphyEmbedShellMode;
 
 struct _EphyEmbedShellClass
