@@ -30,6 +30,7 @@
 #include "ephy-string.h"
 #include "ephy-view-source-handler.h"
 
+#include <evince-document.h>
 #include <glib/gi18n.h>
 #include <jsc/jsc.h>
 #include <libsoup/soup.h>
@@ -304,11 +305,35 @@ ephy_embed_utils_autosearch_address (const char *search_key)
   return effective_address;
 }
 
+static char *
+ensure_host_name_is_lowercase (const char *address)
+{
+  g_autofree gchar *host = ephy_string_get_host_name (address);
+  g_autofree gchar *lowercase_host = NULL;
+  char *ret = NULL;
+
+  if (host == NULL) {
+    return g_strdup (address);
+  }
+
+  lowercase_host = g_utf8_strdown (host, -1);
+
+  if (strcmp (host, lowercase_host) != 0) {
+    ret = ephy_string_find_and_replace (address, host, lowercase_host);
+  } else {
+    ret = g_strdup (address);
+  }
+
+  return ret;
+}
+
 char *
 ephy_embed_utils_normalize_or_autosearch_address (const char *address)
 {
-  if (ephy_embed_utils_address_is_valid (address))
-    return ephy_embed_utils_normalize_address (address);
+  g_autofree gchar *lower_case_address = ensure_host_name_is_lowercase (address);
+
+  if (ephy_embed_utils_address_is_valid (lower_case_address))
+    return ephy_embed_utils_normalize_address (lower_case_address);
   else
     return ephy_embed_utils_autosearch_address (address);
 }
@@ -391,3 +416,27 @@ ephy_embed_utils_shutdown (void)
   g_clear_pointer (&non_search_regex, g_regex_unref);
   g_clear_pointer (&domain_regex, g_regex_unref);
 }
+
+gboolean
+ephy_embed_utils_mime_type_is_supported_evince_document (const char *mime_type)
+{
+  GList *doc_types = ev_backends_manager_get_all_types_info ();
+  GList *l;
+  gboolean found = FALSE;
+
+  for (l = doc_types; l != NULL && !found; l = l->next) {
+    EvTypeInfo *info = (EvTypeInfo *)l->data;
+    guint i;
+
+    for (i = 0; info->mime_types[i] != NULL; ++i) {
+      if (g_ascii_strcasecmp (mime_type, info->mime_types[i]) == 0) {
+        found = TRUE;
+        break;
+      }
+    }
+  }
+
+  g_list_free (doc_types);
+
+  return found;
+}
