@@ -29,6 +29,7 @@
 #include "ephy-embed-prefs.h"
 #include "ephy-embed-shell.h"
 #include "ephy-embed-utils.h"
+#include "ephy-evince-document-view.h"
 #include "ephy-find-toolbar.h"
 #include "ephy-notification-container.h"
 #include "ephy-prefs.h"
@@ -65,6 +66,7 @@ struct _EphyEmbed {
   GtkWidget *floating_bar;
   GtkWidget *progress;
   GtkWidget *fullscreen_message_label;
+  GtkWidget *document_view;
 
   char *title;
   WebKitURIRequest *delayed_request;
@@ -74,6 +76,8 @@ struct _EphyEmbed {
   GSList *messages;
   GSList *keys;
 
+  EphyEmbedMode mode;
+
   guint seq_context_id;
   guint seq_message_id;
 
@@ -750,6 +754,7 @@ ephy_embed_constructed (GObject *object)
   gtk_widget_set_no_show_all (embed->floating_bar, TRUE);
 
   gtk_overlay_add_overlay (GTK_OVERLAY (embed->overlay), embed->floating_bar);
+  gtk_overlay_set_overlay_pass_through (GTK_OVERLAY (embed->overlay), embed->floating_bar, TRUE);
 
   if (embed->progress_bar_enabled) {
     embed->progress = gtk_progress_bar_new ();
@@ -807,6 +812,21 @@ ephy_embed_constructed (GObject *object)
   g_signal_connect (inspector, "closed",
                     G_CALLBACK (ephy_embed_close_inspector_cb),
                     embed);
+
+  if (webkit_web_view_is_controlled_by_automation (embed->web_view)) {
+    GtkWidget *info_bar;
+    GtkWidget *label;
+
+    info_bar = gtk_info_bar_new ();
+    gtk_info_bar_set_message_type (GTK_INFO_BAR (info_bar), GTK_MESSAGE_INFO);
+    /* Translators: this means WebDriver control. */
+    label = gtk_label_new (_("Web is being controlled by automation"));
+    gtk_box_pack_start (GTK_BOX (gtk_info_bar_get_content_area (GTK_INFO_BAR (info_bar))), label, FALSE, FALSE, 0);
+    gtk_widget_show (label);
+
+    ephy_embed_add_top_widget (embed, info_bar, EPHY_EMBED_TOP_WIDGET_POLICY_RETAIN_ON_TRANSITION);
+    gtk_widget_show (info_bar);
+  }
 }
 
 static void
@@ -1002,3 +1022,61 @@ ephy_embed_detach_notification_container (EphyEmbed *embed)
     gtk_container_remove (GTK_CONTAINER (embed->overlay), g_object_ref (GTK_WIDGET (container)));
   }
 }
+
+void
+ephy_embed_set_mode (EphyEmbed *embed, EphyEmbedMode mode)
+{
+  g_assert (EPHY_IS_EMBED (embed));
+
+  if (embed->mode == mode)
+    return;
+
+  switch (mode) {
+  case EPHY_EMBED_MODE_WEB_VIEW:
+    if (embed->document_view != NULL) {
+      gtk_widget_destroy (embed->document_view);
+      embed->document_view = NULL;
+    }
+    gtk_widget_set_visible (GTK_WIDGET (embed->paned), TRUE);
+    break;
+  case EPHY_EMBED_MODE_EVINCE_DOCUMENT:
+    gtk_widget_set_visible (GTK_WIDGET (embed->paned), FALSE);
+    embed->document_view = ephy_evince_document_view_new ();
+    ephy_evince_document_set_embed (EPHY_EVINCE_DOCUMENT_VIEW (embed->document_view), embed);
+    gtk_box_pack_start (GTK_BOX (embed),
+                        embed->document_view,
+                        TRUE, TRUE, 0);
+    gtk_widget_show_all (embed->document_view);
+    break;
+  }
+
+  embed->mode = mode;
+}
+
+EphyEmbedMode
+ephy_embed_get_mode (EphyEmbed *embed)
+{
+  return embed->mode;
+}
+
+static void
+document_download_finished_cb (WebKitDownload *download,
+                               EphyEmbed      *embed)
+{
+  const char *document_uri = webkit_download_get_destination (download);
+
+  ephy_evince_document_view_load_uri (EPHY_EVINCE_DOCUMENT_VIEW (embed->document_view),
+                                      document_uri);
+}
+
+void
+ephy_embed_download_started (EphyEmbed    *embed,
+                             EphyDownload *ephy_download)
+{
+  WebKitDownload *download = ephy_download_get_webkit_download (ephy_download);
+
+  if (embed->mode == EPHY_EMBED_MODE_EVINCE_DOCUMENT) {
+    ephy_download_enable_evince_document_mode (ephy_download);
+    g_signal_connect (download, "finished", G_CALLBACK (document_download_finished_cb), embed);
+  }
+}
