@@ -74,6 +74,7 @@ typedef enum
 #define EPHY_PREFS_UI_EXPAND_TABS_BAR            "expand-tabs-bar"
 #define EPHY_PREFS_UI_TABS_BAR_POSITION          "tabs-bar-position"
 #define EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY "tabs-bar-visibility-policy"
+#define EPHY_PREFS_UI_KEEP_WINDOW_OPEN           "keep-window-open"
 
 #define EPHY_PREFS_READER_SCHEMA                 "org.gnome.Epiphany.reader"
 #define EPHY_PREFS_READER_FONT_STYLE             "font-style"
@@ -119,6 +120,9 @@ static const char * const ephy_prefs_state_schema[] = {
 #define EPHY_PREFS_WEB_GSB_API_KEY                 "gsb-api-key"
 #define EPHY_PREFS_WEB_DEFAULT_ZOOM_LEVEL          "default-zoom-level"
 #define EPHY_PREFS_WEB_ENABLE_AUTOSEARCH           "enable-autosearch"
+#define EPHY_PREFS_WEB_ENABLE_MOUSE_GESTURES       "enable-mouse-gestures"
+#define EPHY_PREFS_WEB_LAST_UPLOAD_DIRECTORY       "last-upload-directory"
+#define EPHY_PREFS_WEB_LAST_DOWNLOAD_DIRECTORY     "last-download-directory"
 
 static const char * const ephy_prefs_web_schema[] = {
   EPHY_PREFS_WEB_FONT_MIN_SIZE,
@@ -144,7 +148,10 @@ static const char * const ephy_prefs_web_schema[] = {
   EPHY_PREFS_WEB_ENABLE_SAFE_BROWSING,
   EPHY_PREFS_WEB_GSB_API_KEY,
   EPHY_PREFS_WEB_DEFAULT_ZOOM_LEVEL,
-  EPHY_PREFS_WEB_ENABLE_AUTOSEARCH
+  EPHY_PREFS_WEB_ENABLE_AUTOSEARCH,
+  EPHY_PREFS_WEB_ENABLE_MOUSE_GESTURES,
+  EPHY_PREFS_WEB_LAST_UPLOAD_DIRECTORY,
+  EPHY_PREFS_WEB_LAST_DOWNLOAD_DIRECTORY,
 };
 
 #define EPHY_PREFS_SCHEMA                             "org.gnome.Epiphany"
