@@ -28,6 +28,7 @@
 #include "ephy-widgets-type-builtins.h"
 #include "ephy-about-handler.h"
 #include "ephy-debug.h"
+#include "ephy-embed-shell.h"
 #include "ephy-gui.h"
 #include "ephy-lib-type-builtins.h"
 #include "ephy-signal-accumulator.h"
@@ -39,12 +40,9 @@
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <string.h>
-#if 0
-/* FIXME: Refactor the DNS prefetch, this is a layering violation */
 #include <libsoup/soup.h>
+#include <string.h>
 #include <webkit2/webkit2.h>
-#endif
 
 /**
  * SECTION:ephy-location-entry
@@ -121,6 +119,7 @@ enum signalsEnum {
 static gint signals[LAST_SIGNAL] = { 0 };
 
 static void ephy_location_entry_title_widget_interface_init (EphyTitleWidgetInterface *iface);
+static void schedule_dns_prefetch (EphyLocationEntry *entry, const gchar *url);
 
 G_DEFINE_TYPE_WITH_CODE (EphyLocationEntry, ephy_location_entry, GTK_TYPE_OVERLAY,
                          G_IMPLEMENT_INTERFACE (EPHY_TYPE_TITLE_WIDGET,
@@ -319,18 +318,6 @@ ephy_location_entry_finalize (GObject *object)
   G_OBJECT_CLASS (ephy_location_entry_parent_class)->finalize (object);
 }
 
-static void
-ephy_location_entry_get_preferred_width (GtkWidget *widget,
-                                         gint      *minimum_width,
-                                         gint      *natural_width)
-{
-  if (minimum_width)
-    *minimum_width = -1;
-
-  if (natural_width)
-    *natural_width = 848;
-}
-
 static void
 ephy_location_entry_get_preferred_height (GtkWidget *widget,
                                           gint      *minimum_height,
@@ -425,7 +412,6 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
   object_class->finalize = ephy_location_entry_finalize;
   object_class->dispose = ephy_location_entry_dispose;
 
-  widget_class->get_preferred_width = ephy_location_entry_get_preferred_width;
   widget_class->get_preferred_height = ephy_location_entry_get_preferred_height;
 
   g_object_class_override_property (object_class, PROP_ADDRESS, "address");
@@ -743,6 +729,14 @@ button_box_size_allocated_cb (GtkWidget    *widget,
   g_free (css);
 }
 
+static gboolean
+event_button_press_event_cb (GtkWidget *widget,
+                             GdkEvent  *event,
+                             gpointer   user_data)
+{
+  return GDK_EVENT_STOP;
+}
+
 static void
 ephy_location_entry_suggestion_activated (DzlSuggestionEntry *entry,
                                           DzlSuggestion      *arg1,
@@ -767,17 +761,21 @@ suggestion_selected (DzlSuggestionEntry *entry,
                      DzlSuggestion      *suggestion,
                      gpointer            user_data)
 {
+  EphyLocationEntry *lentry = EPHY_LOCATION_ENTRY (user_data);
   const gchar *uri = dzl_suggestion_get_id (suggestion);
 
   g_signal_handlers_block_by_func (entry, G_CALLBACK (editable_changed_cb), user_data);
   gtk_entry_set_text (GTK_ENTRY (entry), uri);
   gtk_editable_set_position (GTK_EDITABLE (entry), -1);
   g_signal_handlers_unblock_by_func (entry, G_CALLBACK (editable_changed_cb), user_data);
+
+  schedule_dns_prefetch (lentry, uri);
 }
 
 static void
 ephy_location_entry_construct_contents (EphyLocationEntry *entry)
 {
+  GtkWidget *event;
   GtkWidget *button_box;
   GtkStyleContext *context;
 
@@ -799,15 +797,22 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
   gtk_widget_show (GTK_WIDGET (entry->url_entry));
   gtk_overlay_add_overlay (GTK_OVERLAY (entry), GTK_WIDGET (entry->url_entry));
 
+  /* Event box */
+  event = gtk_event_box_new ();
+  gtk_widget_set_halign (event, GTK_ALIGN_END);
+  gtk_widget_show (event);
+  g_signal_connect (G_OBJECT (event), "button-press-event", G_CALLBACK (event_button_press_event_cb), entry);
+  gtk_overlay_add_overlay (GTK_OVERLAY (entry), event);
+
   /* Button Box */
   button_box = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
+  gtk_container_add (GTK_CONTAINER (event), button_box);
   gtk_box_set_homogeneous (GTK_BOX (button_box), FALSE);
   g_signal_connect (G_OBJECT (button_box), "size-allocate", G_CALLBACK (button_box_size_allocated_cb), entry);
   gtk_button_box_set_layout (GTK_BUTTON_BOX (button_box), GTK_BUTTONBOX_EXPAND);
   gtk_widget_set_halign (button_box, GTK_ALIGN_END);
   gtk_widget_set_margin_end (button_box, 5);
   gtk_widget_show (button_box);
-  gtk_overlay_add_overlay (GTK_OVERLAY (entry), button_box);
 
   /* Bookmark */
   entry->bookmark_event_box = gtk_event_box_new ();
@@ -859,8 +864,6 @@ ephy_location_entry_new (void)
   return GTK_WIDGET (g_object_new (EPHY_TYPE_LOCATION_ENTRY, NULL));
 }
 
-#if 0
-/* FIXME: Refactor the DNS prefetch, this is a layering violation */
 typedef struct {
   SoupURI *uri;
   EphyLocationEntry *entry;
@@ -887,32 +890,67 @@ do_dns_prefetch (PrefetchHelper *helper)
   return FALSE;
 }
 
+/*
+ * Note: As we do not have access to WebKitNetworkProxyMode, and because
+ * Epiphany does not ever change it, we are just checking system default proxy.
+ */
 static void
-schedule_dns_prefetch (EphyLocationEntry *entry, guint interval, const gchar *url)
+proxy_resolver_ready_cb (GObject      *object,
+                         GAsyncResult *result,
+                         gpointer      user_data)
 {
+  PrefetchHelper *helper = user_data;
+  GProxyResolver *resolver = G_PROXY_RESOLVER (object);
+  GError *error = NULL;
+  g_auto(GStrv) proxies = NULL;
+
+  proxies = g_proxy_resolver_lookup_finish (resolver, result, &error);
+  if (error != NULL) {
+    g_error_free (error);
+    free_prefetch_helper (helper);
+    return;
+  }
+
+  if (proxies != NULL && (g_strv_length (proxies) > 1 || g_strcmp0 (proxies[0], "direct://") != 0)) {
+    free_prefetch_helper (helper);
+    return;
+  }
+
+  if (helper->entry->dns_prefetch_handler)
+    g_source_remove (helper->entry->dns_prefetch_handler);
+
+  helper->entry->dns_prefetch_handler =
+    g_timeout_add_full (G_PRIORITY_DEFAULT,
+                        250,
+                        (GSourceFunc)do_dns_prefetch,
+                        helper,
+                        (GDestroyNotify)free_prefetch_helper);
+  g_source_set_name_by_id (helper->entry->dns_prefetch_handler, "[epiphany] do_dns_prefetch");
+}
+
+static void
+schedule_dns_prefetch (EphyLocationEntry *entry,
+                       const gchar       *url)
+{
+  GProxyResolver *resolver = g_proxy_resolver_get_default ();
   PrefetchHelper *helper;
   SoupURI *uri;
 
+  if (resolver == NULL)
+    return;
+
   uri = soup_uri_new (url);
   if (!uri || !uri->host) {
     soup_uri_free (uri);
     return;
   }
 
-  if (entry->dns_prefetch_handler)
-    g_source_remove (entry->dns_prefetch_handler);
-
   helper = g_new0 (PrefetchHelper, 1);
   helper->entry = g_object_ref (entry);
   helper->uri = uri;
 
-  entry->dns_prefetch_handler =
-    g_timeout_add_full (G_PRIORITY_DEFAULT, interval,
-                        (GSourceFunc)do_dns_prefetch, helper,
-                        (GDestroyNotify)free_prefetch_helper);
-  g_source_set_name_by_id (entry->dns_prefetch_handler, "[epiphany] do_dns_prefetch");
+  g_proxy_resolver_lookup_async (resolver, url, NULL, proxy_resolver_ready_cb, helper);
 }
-#endif
 
 /**
  * ephy_location_entry_get_can_undo:
