@@ -23,6 +23,8 @@
 
 #include "ephy-downloads-popover.h"
 #include "ephy-downloads-progress-icon.h"
+#include "ephy-embed.h"
+#include "ephy-embed-container.h"
 #include "ephy-shell.h"
 #include "ephy-window.h"
 
@@ -30,8 +32,6 @@ struct _EphyActionBarEnd {
   GtkBox parent_instance;
 
   GtkWidget *bookmarks_button;
-  GtkWidget *new_tab_revealer;
-  GtkWidget *new_tab_button;
   GtkWidget *downloads_revealer;
   GtkWidget *downloads_button;
   GtkWidget *downloads_popover;
@@ -52,11 +52,27 @@ is_for_active_window (EphyActionBarEnd *action_bar_end)
   return active_window == GTK_WINDOW (ancestor);
 }
 
+static gboolean
+is_document_view_active (void)
+{
+  EphyShell *shell = ephy_shell_get_default ();
+  GtkWindow *window;
+  EphyEmbed *embed;
+
+  window = gtk_application_get_active_window (GTK_APPLICATION (shell));
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+
+  return ephy_embed_get_mode (embed) == EPHY_EMBED_MODE_EVINCE_DOCUMENT;
+}
+
 static void
 download_added_cb (EphyDownloadsManager *manager,
                    EphyDownload         *download,
                    EphyActionBarEnd     *action_bar_end)
 {
+  if (is_document_view_active ())
+    return;
+
   if (!action_bar_end->downloads_popover) {
     action_bar_end->downloads_popover = ephy_downloads_popover_new (action_bar_end->downloads_button);
     gtk_menu_button_set_popover (GTK_MENU_BUTTON (action_bar_end->downloads_button),
@@ -107,12 +123,6 @@ ephy_action_bar_end_class_init (EphyActionBarEndClass *klass)
   gtk_widget_class_bind_template_child (widget_class,
                                         EphyActionBarEnd,
                                         bookmarks_button);
-  gtk_widget_class_bind_template_child (widget_class,
-                                        EphyActionBarEnd,
-                                        new_tab_revealer);
-  gtk_widget_class_bind_template_child (widget_class,
-                                        EphyActionBarEnd,
-                                        new_tab_button);
   gtk_widget_class_bind_template_child (widget_class,
                                         EphyActionBarEnd,
                                         downloads_revealer);
@@ -171,14 +181,6 @@ ephy_action_bar_end_set_show_bookmarks_button (EphyActionBarEnd *action_bar_end,
   gtk_widget_set_visible (action_bar_end->bookmarks_button, show);
 }
 
-void
-ephy_action_bar_end_set_show_new_tab_button (EphyActionBarEnd *action_bar_end,
-                                             gboolean          show)
-{
-  gtk_widget_set_visible (action_bar_end->new_tab_revealer, show);
-  gtk_revealer_set_reveal_child (GTK_REVEALER (action_bar_end->new_tab_revealer), show);
-}
-
 GtkWidget *
 ephy_action_bar_end_get_bookmarks_button (EphyActionBarEnd *action_bar_end)
 {
