@@ -42,6 +42,7 @@ struct _EphyActionBarStart {
   GtkWidget *combined_stop_reload_button;
   GtkWidget *combined_stop_reload_image;
   GtkWidget *homepage_button;
+  GtkWidget *new_tab_button;
 
   guint navigation_buttons_menu_timeout;
 };
@@ -377,6 +378,29 @@ navigation_button_release_event_cb (GtkButton *button,
   return G_SOURCE_REMOVE;
 }
 
+static gboolean
+homepage_button_release_event_cb (GtkButton *button,
+                                  GdkEvent  *event,
+                                  gpointer   user_data)
+{
+  EphyActionBarStart *action_bar_start = EPHY_ACTION_BAR_START (user_data);
+  GActionGroup *action_group;
+  GAction *action;
+
+  action_group = gtk_widget_get_action_group (gtk_widget_get_ancestor (GTK_WIDGET (action_bar_start), EPHY_TYPE_WINDOW), "toolbar");
+
+  switch (((GdkEventButton *)event)->button) {
+    case GDK_BUTTON_MIDDLE:
+      action = g_action_map_lookup_action (G_ACTION_MAP (action_group), "homepage-new-tab");
+       g_action_activate (action, NULL);
+      break;
+    default:
+      break;
+  }
+
+  return G_SOURCE_REMOVE;
+}
+
 static gboolean
 navigation_leave_notify_event_cb (GtkButton *button,
                                   GdkEvent  *event,
@@ -426,8 +450,6 @@ ephy_action_bar_start_constructed (GObject *object)
   EphyActionBarStart *action_bar_start = EPHY_ACTION_BAR_START (object);
   EphyEmbedShell *embed_shell;
 
-  G_OBJECT_CLASS (ephy_action_bar_start_parent_class)->constructed (object);
-
   gtk_widget_init_template (GTK_WIDGET (action_bar_start));
 
   /* Back */
@@ -460,6 +482,8 @@ ephy_action_bar_start_constructed (GObject *object)
   } else {
     gtk_widget_set_visible (action_bar_start->homepage_button, FALSE);
   }
+  g_signal_connect (action_bar_start->homepage_button, "button-release-event",
+                    G_CALLBACK (homepage_button_release_event_cb), action_bar_start);
 }
 
 static void
@@ -492,6 +516,9 @@ ephy_action_bar_start_class_init (EphyActionBarStartClass *klass)
   gtk_widget_class_bind_template_child (widget_class,
                                         EphyActionBarStart,
                                         homepage_button);
+  gtk_widget_class_bind_template_child (widget_class,
+                                        EphyActionBarStart,
+                                        new_tab_button);
 }
 
 static void
