@@ -33,9 +33,12 @@
 #include "ephy-web-app-utils.h"
 
 #include <errno.h>
+#include <evince-document.h>
 #include <glib/gi18n.h>
 #include <glib-unix.h>
 #include <gtk/gtk.h>
+#define HANDY_USE_UNSTABLE_API
+#include <handy.h>
 #include <libnotify/notify.h>
 #include <libxml/xmlreader.h>
 #include <libxml/xmlversion.h>
@@ -43,18 +46,16 @@
 #include <string.h>
 #include <stdlib.h>
 
-static gboolean open_in_new_tab = FALSE;
 static gboolean open_in_new_window = FALSE;
 
 static char *session_filename = NULL;
-static char *bookmark_url = NULL;
-static char *bookmarks_file = NULL;
 static char **arguments = NULL;
 static char *application_to_delete = NULL;
 
 static gboolean private_instance = FALSE;
 static gboolean incognito_mode = FALSE;
 static gboolean application_mode = FALSE;
+static gboolean automation_mode = FALSE;
 static char *desktop_file_basename = NULL;
 static char *profile_directory = NULL;
 
@@ -109,30 +110,26 @@ option_version_cb (const gchar *option_name,
 /* If you're modifying this array then you need to update the manpage. */
 static const GOptionEntry option_entries[] =
 {
-  { "new-tab", 'n', 0, G_OPTION_ARG_NONE, &open_in_new_tab,
-    N_("Open a new tab in an existing browser window"), NULL },
   { "new-window", 0, 0, G_OPTION_ARG_NONE, &open_in_new_window,
-    N_("Open a new browser window"), NULL },
-  { "import-bookmarks", '\0', 0, G_OPTION_ARG_FILENAME, &bookmarks_file,
-    N_("Import bookmarks from the given file"), N_("FILE") },
+    N_("Open a new browser window instead of a new tab"), NULL },
   { "load-session", 'l', 0, G_OPTION_ARG_FILENAME, &session_filename,
-    N_("Load the given session file"), N_("FILE") },
-  { "add-bookmark", 't', 0, G_OPTION_ARG_STRING, &bookmark_url,
-    N_("Add a bookmark"), N_("URL") },
-  { "private-instance", 'p', 0, G_OPTION_ARG_NONE, &private_instance,
-    N_("Start a private instance"), NULL },
+    N_("Load the given session state file"), N_("FILE") },
   { "incognito-mode", 'i', 0, G_OPTION_ARG_NONE, &incognito_mode,
-    N_("Start an instance in incognito mode"), NULL },
+    N_("Start an instance with user data read-only"), NULL },
+  { "private-instance", 'p', 0, G_OPTION_ARG_NONE, &private_instance,
+    N_("Start a private instance with separate user data"), NULL },
   { "application-mode", 'a', G_OPTION_FLAG_FILENAME | G_OPTION_FLAG_OPTIONAL_ARG,
     G_OPTION_ARG_CALLBACK, application_mode_cb,
-    N_("Start the browser in application mode"), NULL },
+    N_("Start a private instance in web application mode"), NULL },
+  { "automation-mode", 0, 0, G_OPTION_ARG_NONE, &automation_mode,
+    N_("Start a private instance for WebDriver control"), NULL },
   { "profile", 0, 0, G_OPTION_ARG_STRING, &profile_directory,
-    N_("Profile directory to use in the private instance"), N_("DIR") },
+    N_("Custom profile directory for private instance"), N_("DIR") },
   { G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &arguments,
     "", N_("URL …") },
   { "version", 0, G_OPTION_FLAG_NO_ARG | G_OPTION_FLAG_HIDDEN,
     G_OPTION_ARG_CALLBACK, option_version_cb, NULL, NULL },
-  { "delete-application", 0, 0, G_OPTION_ARG_STRING | G_OPTION_FLAG_HIDDEN,
+  { "delete-application", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING,
     &application_to_delete, NULL, NULL },
   { NULL }
 };
@@ -168,19 +165,6 @@ get_startup_id (void)
   return retval;
 }
 
-static EphyStartupFlags
-get_startup_flags (void)
-{
-  EphyStartupFlags flags = 0;
-
-  if (open_in_new_tab)
-    flags |= EPHY_STARTUP_NEW_TAB;
-  if (open_in_new_window)
-    flags |= EPHY_STARTUP_NEW_WINDOW;
-
-  return flags;
-}
-
 int
 main (int   argc,
       char *argv[])
@@ -191,12 +175,15 @@ main (int   argc,
   guint32 user_time;
   gboolean arbitrary_url;
   EphyShellStartupContext *ctx;
-  EphyStartupFlags startup_flags;
   EphyEmbedShellMode mode;
   int status;
   EphyFileHelpersFlags flags;
   GDesktopAppInfo *desktop_info = NULL;
 
+#if DEVELOPER_MODE
+  g_setenv ("GSETTINGS_SCHEMA_DIR", BUILD_ROOT "/data", FALSE);
+#endif
+
   /* Initialize the i18n stuff */
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -300,6 +287,11 @@ main (int   argc,
     exit (1);
   }
 
+  if (automation_mode && (private_instance || incognito_mode || application_mode || profile_directory)) {
+    g_print ("Cannot use --automation-mode and --private-instance, --incognito-mode, --application-mode, or --profile at the same time\n");
+    exit (1);
+  }
+
   if (application_mode && profile_directory && !g_file_test (profile_directory, G_FILE_TEST_IS_DIR)) {
     g_print ("--profile must be an existing directory when --application-mode is requested\n");
     exit (1);
@@ -330,7 +322,7 @@ main (int   argc,
   /* Start our services */
   flags = !application_mode ? EPHY_FILE_HELPERS_ENSURE_EXISTS : 0;
 
-  if (incognito_mode || private_instance || application_mode)
+  if (incognito_mode || private_instance || application_mode || automation_mode)
     flags |= EPHY_FILE_HELPERS_PRIVATE_PROFILE;
   if (incognito_mode)
     flags |= EPHY_FILE_HELPERS_STEAL_DATA;
@@ -343,8 +335,8 @@ main (int   argc,
 
   /* Run the migration in all cases, except when running a private
      instance without a given profile directory or running in
-     incognito mode. */
-  if (!(private_instance && profile_directory == NULL) && incognito_mode == FALSE) {
+     incognito or automation mode. */
+  if (!(private_instance && profile_directory == NULL) && !incognito_mode && !automation_mode) {
     /* If the migration fails we don't really want to continue. */
     if (!ephy_profile_utils_do_migration ((const char *)profile_directory, -1, FALSE)) {
       g_print ("Failed to run the migrator process, Web will now abort.\n");
@@ -352,6 +344,10 @@ main (int   argc,
     }
   }
 
+  /* Ignore arguments in automation mode */
+  if (automation_mode)
+    g_clear_pointer (&arguments, g_strfreev);
+
   arbitrary_url = g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
                                           EPHY_PREFS_LOCKDOWN_ARBITRARY_URL);
 
@@ -381,16 +377,13 @@ main (int   argc,
     exit (0);
   }
 
-  startup_flags = get_startup_flags ();
-
   /* Now create the shell */
   if (private_instance) {
     mode = EPHY_EMBED_SHELL_MODE_PRIVATE;
-    /* In private mode the session autoresume will always open an empty window.
-     * If there are arguments, we want the URIs to be opened in thet existing window. */
-    startup_flags |= EPHY_STARTUP_NEW_TAB;
   } else if (incognito_mode) {
     mode = EPHY_EMBED_SHELL_MODE_INCOGNITO;
+  } else if (automation_mode) {
+    mode = EPHY_EMBED_SHELL_MODE_AUTOMATION;
   } else if (application_mode) {
     mode = EPHY_EMBED_SHELL_MODE_APPLICATION;
 
@@ -415,12 +408,13 @@ main (int   argc,
     gtk_window_set_default_icon_name ("org.gnome.Epiphany");
   }
 
+  hdy_init (&argc, &argv);
+  ev_init ();
+
   _ephy_shell_create_instance (mode);
 
-  ctx = ephy_shell_startup_context_new (startup_flags,
-                                        bookmarks_file,
+  ctx = ephy_shell_startup_context_new (open_in_new_window ? EPHY_STARTUP_NEW_WINDOW : EPHY_STARTUP_NEW_TAB,
                                         session_filename,
-                                        bookmark_url,
                                         arguments,
                                         user_time);
   g_strfreev (arguments);
@@ -441,6 +435,7 @@ main (int   argc,
   if (notify_is_initted ())
     notify_uninit ();
 
+  ev_shutdown ();
   ephy_settings_shutdown ();
   ephy_file_helpers_shutdown ();
   xmlCleanupParser ();
