@@ -56,6 +56,7 @@
 #include <gtk/gtk.h>
 #include <JavaScriptCore/JavaScript.h>
 #include <json-glib/json-glib.h>
+#include <math.h>
 #include <string.h>
 
 #define DOWNLOAD_BUTTON_WIDTH   8
@@ -88,11 +89,13 @@ struct _PrefsDialog {
   GtkWidget *download_box;
   GtkWidget *search_box;
   GtkWidget *session_box;
+  GtkWidget *browsing_box;
   GtkWidget *restore_session_checkbutton;
   GtkWidget *popups_allow_checkbutton;
   GtkWidget *adblock_allow_checkbutton;
   GtkWidget *enable_plugins_checkbutton;
   GtkWidget *enable_safe_browsing_checkbutton;
+  GtkWidget *enable_smooth_scrolling_checkbutton;
 
   /* fonts & style */
   GtkWidget *use_gnome_fonts_checkbutton;
@@ -102,6 +105,7 @@ struct _PrefsDialog {
   GtkWidget *mono_fontbutton;
   GtkWidget *css_checkbox;
   GtkWidget *css_edit_button;
+  GtkWidget *default_zoom_spin_button;
   GtkWidget *reader_mode_box;
   GtkWidget *reader_mode_font_style;
   GtkWidget *reader_mode_color_scheme;
@@ -925,6 +929,35 @@ on_search_engine_dialog_button_clicked (GtkWidget   *button,
   gtk_window_present (search_engine_dialog);
 }
 
+static gboolean
+on_default_zoom_spin_button_output (GtkSpinButton *spin,
+                                    gpointer       user_data)
+{
+  GtkAdjustment *adjustment;
+  g_autofree gchar *text = NULL;
+  gdouble value;
+
+  adjustment = gtk_spin_button_get_adjustment (spin);
+  value = (int)gtk_adjustment_get_value (adjustment);
+  text = g_strdup_printf ("%.f%%", value);
+  gtk_entry_set_text (GTK_ENTRY (spin), text);
+
+  return TRUE;
+}
+
+static void
+on_default_zoom_spin_button_value_changed (GtkSpinButton *spin,
+                                           gpointer       user_data)
+{
+  GtkAdjustment *adjustment;
+  gdouble value;
+
+  adjustment = gtk_spin_button_get_adjustment (spin);
+  value = gtk_adjustment_get_value (adjustment);
+  value = roundf(value) / 100;
+  g_settings_set_double (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DEFAULT_ZOOM_LEVEL, value);
+}
+
 static void
 prefs_dialog_class_init (PrefsDialogClass *klass)
 {
@@ -950,10 +983,12 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, webapp_title);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, search_box);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, session_box);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, browsing_box);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, restore_session_checkbutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, popups_allow_checkbutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, adblock_allow_checkbutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, enable_safe_browsing_checkbutton);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, enable_smooth_scrolling_checkbutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, download_button_hbox);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, download_button_label);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, download_box);
@@ -966,6 +1001,7 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, mono_fontbutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, css_checkbox);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, css_edit_button);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, default_zoom_spin_button);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, reader_mode_box);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, reader_mode_font_style);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, reader_mode_color_scheme);
@@ -1023,6 +1059,8 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, on_sync_device_name_change_button_clicked);
   gtk_widget_class_bind_template_callback (widget_class, on_sync_device_name_save_button_clicked);
   gtk_widget_class_bind_template_callback (widget_class, on_sync_device_name_cancel_button_clicked);
+  gtk_widget_class_bind_template_callback (widget_class, on_default_zoom_spin_button_output);
+  gtk_widget_class_bind_template_callback (widget_class, on_default_zoom_spin_button_value_changed);
 }
 
 static void
@@ -1981,6 +2019,11 @@ setup_general_page (PrefsDialog *dialog)
                     "clicked",
                     G_CALLBACK (do_not_track_button_clicked_cb),
                     dialog);
+  g_settings_bind (web_settings,
+                   EPHY_PREFS_WEB_ENABLE_SMOOTH_SCROLLING,
+                   dialog->enable_smooth_scrolling_checkbutton,
+                   "active",
+                   G_SETTINGS_BIND_DEFAULT);
 
   if (ephy_is_running_inside_flatpak ())
     gtk_widget_hide (dialog->download_box);
@@ -2148,6 +2191,9 @@ setup_fonts_page (PrefsDialog *dialog)
                     "clicked",
                     G_CALLBACK (css_edit_button_clicked_cb),
                     dialog);
+
+  gtk_spin_button_set_value (GTK_SPIN_BUTTON (dialog->default_zoom_spin_button),
+                             g_settings_get_double (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DEFAULT_ZOOM_LEVEL) * 100);
 }
 
 static void
@@ -2345,8 +2391,12 @@ prefs_dialog_init (PrefsDialog *dialog)
                           mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
   gtk_widget_set_visible (dialog->session_box,
                           mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
+  gtk_widget_set_visible (dialog->browsing_box,
+                          mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
   gtk_widget_set_visible (dialog->do_not_track_checkbutton,
                           mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
+  gtk_widget_set_visible (dialog->enable_smooth_scrolling_checkbutton,
+                          mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
   gtk_widget_set_visible (dialog->reader_mode_box,
                           mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
 
