@@ -605,7 +605,7 @@ window_cmd_show_about (GSimpleAction *action,
 
   gtk_about_dialog_set_version (dialog, VCSVERSION);
   gtk_about_dialog_set_copyright (dialog, "Copyright © 2002–2004 Marco Pesenti Gritti\n"
-                                          "Copyright © 2003–2018 The GNOME Web Developers");
+                                          "Copyright © 2003–2019 The GNOME Web Developers");
   gtk_about_dialog_set_comments (dialog, comments);
   gtk_about_dialog_set_license_type (dialog, GTK_LICENSE_GPL_3_0);
   gtk_about_dialog_set_website (dialog, "https://wiki.gnome.org/Apps/Web");
@@ -668,7 +668,9 @@ window_cmd_navigation (GSimpleAction *action,
   web_view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
 
   if (strstr (g_action_get_name (G_ACTION (action)), "back")) {
+    ephy_embed_set_mode (embed, EPHY_EMBED_MODE_WEB_VIEW);
     webkit_web_view_go_back (web_view);
+
     gtk_widget_grab_focus (GTK_WIDGET (embed));
   } else {
     webkit_web_view_go_forward (web_view);
@@ -1418,25 +1420,31 @@ get_suggested_filename (EphyEmbed *embed)
   const char *mimetype;
   WebKitURIResponse *response;
   WebKitWebResource *web_resource;
+  SoupURI *soup_uri;
 
   view = ephy_embed_get_web_view (embed);
   web_resource = webkit_web_view_get_main_resource (WEBKIT_WEB_VIEW (view));
   response = webkit_web_resource_get_response (web_resource);
   mimetype = webkit_uri_response_get_mime_type (response);
+  soup_uri = soup_uri_new (webkit_web_resource_get_uri (web_resource));
 
-  if ((g_ascii_strncasecmp (mimetype, "text/html", 9)) == 0) {
+  if (g_ascii_strncasecmp (mimetype, "text/html", 9) == 0 && g_strcmp0 (soup_uri_get_scheme (soup_uri), EPHY_VIEW_SOURCE_SCHEME) != 0) {
     /* Web Title will be used as suggested filename */
     suggested_filename = g_strconcat (ephy_embed_get_title (embed), ".mhtml", NULL);
   } else {
     suggested_filename = g_strdup (webkit_uri_response_get_suggested_filename (response));
     if (!suggested_filename) {
-      SoupURI *soup_uri = soup_uri_new (webkit_web_resource_get_uri (web_resource));
       char *last_slash = strrchr (soup_uri->path, '/');
       suggested_filename = soup_uri_decode (last_slash ? (last_slash + 1) : soup_uri->path);
-      soup_uri_free (soup_uri);
+
+      if (!strlen (suggested_filename)) {
+        g_free (suggested_filename);
+        suggested_filename = g_strdup ("index.html");
+      }
     }
   }
 
+  soup_uri_free (soup_uri);
   return suggested_filename;
 }
 
@@ -1458,6 +1466,8 @@ save_response_cb (GtkNativeDialog *dialog, int response, EphyEmbed *embed)
       g_free (converted);
       g_free (uri);
     }
+
+    g_settings_set_string (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_LAST_DOWNLOAD_DIRECTORY, gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dialog)));
   }
 
   g_object_unref (dialog);
@@ -1482,6 +1492,7 @@ window_cmd_save_as (GSimpleAction *action,
                                      EPHY_FILE_FILTER_NONE);
 
   gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
+  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), g_settings_get_string (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_LAST_DOWNLOAD_DIRECTORY));
 
   suggested_filename = ephy_sanitize_filename (get_suggested_filename (embed));
 
@@ -1734,7 +1745,7 @@ window_cmd_zoom_normal (GSimpleAction *action,
                         gpointer       user_data)
 {
   EphyWindow *window = user_data;
-  ephy_window_set_zoom (window, 1.0);
+  ephy_window_set_zoom (window, 0.0);
 }
 
 void
@@ -2204,13 +2215,16 @@ window_cmd_change_fullscreen_state (GSimpleAction *action,
                                     GVariant      *state,
                                     gpointer       user_data)
 {
+  EphyWindow *window = EPHY_WINDOW (user_data);
   gboolean active;
 
   active = g_variant_get_boolean (state);
+  ephy_window_show_fullscreen_header_bar (window);
+
   if (active)
-    gtk_window_fullscreen (GTK_WINDOW (user_data));
+    gtk_window_fullscreen (GTK_WINDOW (window));
   else
-    gtk_window_unfullscreen (GTK_WINDOW (user_data));
+    gtk_window_unfullscreen (GTK_WINDOW (window));
 
   g_simple_action_set_state (action, g_variant_new_boolean (active));
 }
@@ -2443,6 +2457,57 @@ window_cmd_tabs_close_others (GSimpleAction *action,
   g_slist_free (pages_to_close);
 }
 
+void
+window_cmd_tabs_reload (GSimpleAction *action,
+                        GVariant      *parameter,
+                        gpointer       user_data)
+{
+  EphyWindow *window = user_data;
+  EphyEmbed *embed;
+  WebKitWebView *view;
+
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  g_assert (embed != NULL);
+
+  gtk_widget_grab_focus (GTK_WIDGET (embed));
+
+  view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
+  webkit_web_view_reload (view);
+}
+
+void
+window_cmd_tabs_reload_all_tabs (GSimpleAction *action,
+                                 GVariant      *parameter,
+                                 gpointer       user_data)
+{
+  EphyWindow *window = user_data;
+  EphyEmbed *embed;
+  WebKitWebView *view;
+  GtkWidget *notebook;
+  int n_pages;
+
+  notebook = ephy_window_get_notebook (window);
+  n_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
+
+  for (int i = 0; i < n_pages; i++) {
+    embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
+
+    view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
+    webkit_web_view_reload (view);
+  }
+}
+
+void
+window_cmd_tabs_reopen_closed_tab (GSimpleAction *action,
+                                   GVariant      *parameter,
+                                   gpointer       user_data)
+{
+  EphySession *session = ephy_shell_get_session (ephy_shell_get_default ());
+
+  g_assert (session != NULL);
+  ephy_session_undo_close_tab (session);
+}
+
 void
 window_cmd_show_tab (GSimpleAction *action,
                      GVariant      *parameter,
@@ -2490,3 +2555,45 @@ window_cmd_toggle_reader_mode (GSimpleAction *action,
 
   ephy_web_view_toggle_reader_mode (web_view, !active);
 }
+
+void
+window_cmd_open_application_manager (GSimpleAction *action,
+                                     GVariant      *parameter,
+                                     gpointer       user_data)
+{
+  EphyWindow *window = EPHY_WINDOW (user_data);
+  EphyEmbed *embed;
+  EphyWebView *web_view;
+
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  g_assert (embed != NULL);
+
+  web_view = ephy_embed_get_web_view (embed);
+
+  ephy_web_view_load_url (web_view, "about:applications");
+}
+
+void
+window_cmd_homepage_new_tab (GSimpleAction *action,
+                             GVariant      *parameter,
+                             gpointer       user_data)
+{
+  EphyWindow *window = EPHY_WINDOW (user_data);
+  EphyEmbed *embed;
+  EphyWebView *web_view;
+
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  g_assert (embed != NULL);
+
+  embed = ephy_shell_new_tab (ephy_shell_get_default (),
+                              EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed))),
+                              NULL,
+                              0);
+
+  web_view = ephy_embed_get_web_view (embed);
+  ephy_web_view_load_homepage (web_view);
+
+  ephy_embed_container_set_active_child (EPHY_EMBED_CONTAINER (window), embed);
+
+  gtk_widget_grab_focus (GTK_WIDGET (embed));
+}
